@@ -1,12 +1,12 @@
 import os
 import platform
 import random
-import shlex
 import socket
 import subprocess
 import sys
 import time
 from abc import ABC, abstractmethod
+from functools import cached_property
 from typing import Any, Dict, Optional
 
 import ramalama.chat as chat
@@ -14,8 +14,6 @@ from ramalama.common import (
     MNT_DIR,
     MNT_FILE_DRAFT,
     accel_image,
-    check_metal,
-    check_nvidia,
     exec_cmd,
     genname,
     is_split_file_model,
@@ -24,8 +22,7 @@ from ramalama.common import (
     set_accel_env_vars,
 )
 from ramalama.compose import Compose
-from ramalama.config import CONFIG, DEFAULT_PORT, DEFAULT_PORT_RANGE
-from ramalama.console import should_colorize
+from ramalama.config import CONFIG, DEFAULT_PORT_RANGE
 from ramalama.engine import Engine, dry_run, is_healthy, wait_for_healthy
 from ramalama.kube import Kube
 from ramalama.logger import logger
@@ -36,9 +33,8 @@ from ramalama.model_inspect.safetensor_info import SafetensorModelInfo
 from ramalama.model_inspect.safetensor_parser import SafetensorInfoParser
 from ramalama.model_store.global_store import GlobalModelStore
 from ramalama.model_store.store import ModelStore
+from ramalama.path_utils import get_container_mount_path
 from ramalama.quadlet import Quadlet
-from ramalama.rag import rag_image
-from ramalama.version import version
 
 MODEL_TYPES = ["file", "https", "http", "oci", "huggingface", "hf", "modelscope", "ms", "ollama", "rlcr"]
 
@@ -105,23 +101,23 @@ class TransportBase(ABC):
         raise self.__not_implemented_error("push")
 
     @abstractmethod
-    def remove(self, args):
+    def remove(self, args) -> bool:
         raise self.__not_implemented_error("rm")
 
     @abstractmethod
-    def bench(self, args):
+    def bench(self, args, cmd: list[str]):
         raise self.__not_implemented_error("bench")
 
     @abstractmethod
-    def run(self, args):
+    def run(self, args, server_cmd: list[str]):
         raise self.__not_implemented_error("run")
 
     @abstractmethod
-    def perplexity(self, args):
+    def perplexity(self, args, cmd: list[str]):
         raise self.__not_implemented_error("perplexity")
 
     @abstractmethod
-    def serve(self, args):
+    def serve(self, args, cmd: list[str]):
         raise self.__not_implemented_error("serve")
 
     @abstractmethod
@@ -156,6 +152,11 @@ class Transport(TransportBase):
         self._model_store: Optional[ModelStore] = None
 
         self.default_image = accel_image(CONFIG)
+        self.draft_model: Transport | None = None
+
+    @cached_property
+    def artifact(self) -> bool:
+        return self.is_artifact()
 
     def extract_model_identifiers(self):
         model_name = self.model
@@ -206,32 +207,54 @@ class Transport(TransportBase):
 
         if self.model_type == 'oci':
             if use_container or should_generate:
-                return os.path.join(MNT_DIR, 'model.file')
+                if getattr(self, "artifact", False):
+                    artifact_name_method = getattr(self, "artifact_name", None)
+                    if artifact_name_method:
+                        try:
+                            return f"{MNT_DIR}/{artifact_name_method()}"
+                        except subprocess.CalledProcessError:
+                            pass
+                return f"{MNT_DIR}/model.file"
             else:
                 return f"oci://{self.model}"
 
         ref_file = self.model_store.get_ref_file(self.model_tag)
         if ref_file is None:
             raise NoRefFileFound(self.model)
-        if not ref_file.model_files:
-            if any(file.name.endswith(".safetensors") for file in ref_file.files):
-                raise SafetensorModelNotSupported()
+
+        gguf_files = ref_file.model_files
+        safetensor_files = ref_file.safetensor_model_files
+        if safetensor_files:
+            if use_container or should_generate:
+                return MNT_DIR
+            return self.model_store.get_snapshot_directory_from_tag(self.model_tag)
+        elif not gguf_files:
             raise NoGGUFModelFileFound()
 
         # Use the first model file
+        model_file = gguf_files[0]
         if is_split_file_model(self.model_name):
             # Find model file with index 1 for split models
-            index_models = [file for file in ref_file.model_files if "-00001-of-" in file.name]
+            index_models = [file for file in gguf_files if "-00001-of-" in file.name]
             if len(index_models) != 1:
                 raise Exception(f"Found multiple index 1 gguf models: {index_models}")
             model_file = index_models[0]
-        else:
-            model_file = ref_file.model_files[0]
 
         if use_container or should_generate:
-            return os.path.join(MNT_DIR, model_file.name)
+            return f"{MNT_DIR}/{model_file.name}"
         return self.model_store.get_blob_file_path(model_file.hash)
 
+    def _get_inspect_model_path(self, dry_run: bool) -> str:
+        """Return a concrete file path for inspection.
+        Prefer the safetensor blob if available; otherwise use the entry path.
+        """
+        if dry_run:
+            return "/path/to/model"
+        if self.model_type == 'oci':
+            return self._get_entry_model_path(False, False, dry_run)
+        safetensor_blob = self.model_store.get_safetensor_blob_path(self.model_tag, self.filename)
+        return safetensor_blob or self._get_entry_model_path(False, False, dry_run)
+
     def _get_mmproj_path(self, use_container: bool, should_generate: bool, dry_run: bool) -> Optional[str]:
         """
         Returns the path to the mmproj blob on the host if use_container and should_generate are both False.
@@ -253,7 +276,7 @@ class Transport(TransportBase):
         # Use the first mmproj file
         mmproj_file = ref_file.mmproj_files[0]
         if use_container or should_generate:
-            return os.path.join(MNT_DIR, mmproj_file.name)
+            return f"{MNT_DIR}/{mmproj_file.name}"
         return self.model_store.get_blob_file_path(mmproj_file.hash)
 
     def _get_chat_template_path(self, use_container: bool, should_generate: bool, dry_run: bool) -> Optional[str]:
@@ -277,13 +300,16 @@ class Transport(TransportBase):
         # Use the last chat template file (may have been go template converted to jinja)
         chat_template_file = ref_file.chat_templates[-1]
         if use_container or should_generate:
-            return os.path.join(MNT_DIR, chat_template_file.name)
+            return f"{MNT_DIR}/{chat_template_file.name}"
         return self.model_store.get_blob_file_path(chat_template_file.hash)
 
-    def remove(self, args):
+    def remove(self, args) -> bool:
         _, tag, _ = self.extract_model_identifiers()
-        if not self.model_store.remove_snapshot(tag) and not args.ignore:
+        if self.model_store.remove_snapshot(tag):
+            return True
+        if not args.ignore:
             raise KeyError(f"Model '{self.model}' not found")
+        return False
 
     def get_container_name(self, args):
         if getattr(args, "name", None):
@@ -291,15 +317,14 @@ class Transport(TransportBase):
 
         return genname()
 
+    def new_engine(self, args):
+        return Engine(args)
+
     def base(self, args, name):
-        # force accel_image to use -rag version. Drop TAG if it exists
-        # so that accel_image will add -rag to the image specification.
-        if getattr(args, "rag", None) and not getattr(args, "image_override", False):
-            args.image = rag_image(args.image)
         if self.type == "Ollama":
             args.UNRESOLVED_MODEL = args.MODEL
             args.MODEL = self.resolve_model()
-        self.engine = Engine(args)
+        self.engine = self.new_engine(args)
         if args.subcommand == "run" and not getattr(args, "ARGS", None) and sys.stdin.isatty():
             self.engine.add(["-i"])
 
@@ -318,26 +343,6 @@ class Transport(TransportBase):
         name = self.get_container_name(args)
         self.base(args, name)
 
-    def gpu_args(self, args, runner=False):
-        gpu_args = []
-        if args.ngl < 0:
-            args.ngl = 999
-
-        if runner:
-            gpu_args += ["--ngl"]  # double dash
-        else:
-            gpu_args += ["-ngl"]  # single dash
-
-        gpu_args += [f'{args.ngl}']
-
-        if self.draft_model:
-            # Use the same arg as ngl to reduce configuration space
-            gpu_args += ["-ngld", f'{args.ngl}']
-
-        gpu_args += ["--threads", f"{args.threads}"]
-
-        return gpu_args
-
     def exec_model_in_container(self, cmd_args, args):
         if not args.container:
             return False
@@ -357,12 +362,13 @@ class Transport(TransportBase):
     def setup_mounts(self, args):
         if args.dryrun:
             return
+
         if self.model_type == 'oci':
             if self.engine.use_podman:
-                mount_cmd = f"--mount=type=image,src={self.model},destination={MNT_DIR},subpath=/models,rw=false"
+                mount_cmd = self.mount_cmd()
             elif self.engine.use_docker:
                 output_filename = self._get_entry_model_path(args.container, True, args.dryrun)
-                volume = populate_volume_from_image(self, os.path.basename(output_filename))
+                volume = populate_volume_from_image(self, args, os.path.basename(output_filename))
                 mount_cmd = f"--mount=type=volume,src={volume},dst={MNT_DIR},readonly"
             else:
                 raise NotImplementedError(f"No compatible oci mount method for engine: {self.engine.args.engine}")
@@ -377,73 +383,104 @@ class Transport(TransportBase):
         # mount all files into container with file name instead of hash
         for file in ref_file.files:
             blob_path = self.model_store.get_blob_file_path(file.hash)
-            mount_path = os.path.join(MNT_DIR, file.name)
-            self.engine.add([f"--mount=type=bind,src={blob_path},destination={mount_path},ro{self.engine.relabel()}"])
+            # Convert path to container-friendly format (handles Windows path conversion)
+            container_blob_path = get_container_mount_path(blob_path)
+            mount_path = f"{MNT_DIR}/{file.name}"
+            self.engine.add(
+                [f"--mount=type=bind,src={container_blob_path},destination={mount_path},ro{self.engine.relabel()}"]
+            )
 
         if self.draft_model:
             draft_model = self.draft_model._get_entry_model_path(args.container, args.generate, args.dryrun)
-            self.engine.add(
-                [f"--mount=type=bind,src={draft_model},destination={MNT_FILE_DRAFT},ro{self.engine.relabel()}"]
-            )
+            # Convert path to container-friendly format (handles Windows path conversion)
+            container_draft_model = get_container_mount_path(draft_model)
+            mount_opts = f"--mount=type=bind,src={container_draft_model},destination={MNT_FILE_DRAFT}"
+            mount_opts += f",ro{self.engine.relabel()}"
+            self.engine.add([mount_opts])
 
-    def bench(self, args):
-        self.ensure_model_exists(args)
-        exec_args = self.build_exec_args_bench(args)
-        self.execute_command(exec_args, args)
+    def bench(self, args, cmd: list[str]):
+        set_accel_env_vars()
+        self.execute_command(cmd, args)
 
-    def run(self, args):
+    def run(self, args, server_cmd: list[str]):
         # The Run command will first launch a daemonized service
         # and run chat to communicate with it.
 
-        args.port = compute_serving_port(args, quiet=not args.debug)
+        process = self.serve_nonblocking(args, server_cmd)
+        if process:
+            return self._connect_and_chat(args, process)
+
+    def serve_nonblocking(self, args, cmd: list[str]) -> subprocess.Popen | None:
         if args.container:
             args.name = self.get_container_name(args)
 
-        args.noout = not args.debug
-
-        self.ensure_model_exists(args)
-        pid = os.fork()
-        if pid == 0:
-            # Child process - start the server
-            self._start_server(args)
-            return 0
-        else:
-            # Parent process - connect to server and start chat
-            return self._connect_and_chat(args, pid)
-
-    def _start_server(self, args):
-        """Start the server in the child process."""
+        # Use subprocess.Popen for all platforms
+        # Prepare args for the server
         args.host = CONFIG.host
-        args.generate = ""
         args.detach = True
-        self.serve(args, True)
 
-    def _connect_and_chat(self, args, server_pid):
-        """Connect to the server and start chat in the parent process."""
-        args.url = f"http://127.0.0.1:{args.port}"
-        if getattr(args, "runtime", None) == "mlx":
-            args.url += "/v1"
-            args.prefix = "🍏 > "
-        args.pid2kill = ""
+        set_accel_env_vars()
 
         if args.container:
-            return self._handle_container_chat(args, server_pid)
+            # For container mode, set up the container and start it with subprocess
+            self.setup_container(args)
+            self.setup_mounts(args)
+            # Make sure Image precedes cmd_args
+            self.engine.add([args.image] + cmd)
+
+            if args.dryrun:
+                self.engine.dryrun()
+                return None
+
+            # Start the container using subprocess.Popen
+            process = subprocess.Popen(
+                self.engine.exec_args,
+            )
+            return process
+
+        # Non-container mode: run the command directly with subprocess
+        if args.dryrun:
+            dry_run(cmd)
+            return None
+
+        process = subprocess.Popen(
+            cmd,
+        )
+        return process
+
+    def _connect_and_chat(self, args, server_process):
+        """Connect to the server and start chat in the parent process."""
+        args.url = f"http://127.0.0.1:{args.port}/v1"
+        if getattr(args, "runtime", None) == "mlx":
+            args.prefix = "🍏 > "
+
+        if args.container:
+            return self._handle_container_chat(args, server_process)
         else:
-            args.pid2kill = server_pid
+            # Store the Popen object for monitoring
+            args.server_process = server_process
+
             if getattr(args, "runtime", None) == "mlx":
                 return self._handle_mlx_chat(args)
             chat.chat(args)
             return 0
 
-    def _handle_container_chat(self, args, server_pid):
+    def chat_operational_args(self, args):
+        return None
+
+    def wait_for_healthy(self, args):
+        wait_for_healthy(args, is_healthy)
+
+    def _handle_container_chat(self, args, server_process):
         """Handle chat for container-based execution."""
-        _, status = os.waitpid(server_pid, 0)
-        if status != 0:
+        # Wait for the server process to complete (blocking)
+        exit_code = server_process.wait()
+        if exit_code != 0:
             raise ValueError(f"Failed to serve model {self.model_name}, for ramalama run command")
 
         if not args.dryrun:
             try:
-                wait_for_healthy(args, is_healthy)
+                self.wait_for_healthy(args)
             except subprocess.TimeoutExpired as e:
                 logger.error(f"Failed to serve model {self.model_name}, for ramalama run command")
                 logger.error(f"{e}: logs: {e.output}")
@@ -452,7 +489,7 @@ class Transport(TransportBase):
         args.ignore = getattr(args, "dryrun", False)
         for i in range(6):
             try:
-                chat.chat(args)
+                chat.chat(args, self.chat_operational_args(args))
                 break
             except Exception as e:
                 if i >= 5:
@@ -481,7 +518,7 @@ class Transport(TransportBase):
             except Exception as e:
                 if i >= max_retries - 1:
                     perror(f"Error: Failed to connect to MLX server after {max_retries} attempts: {e}")
-                    self._cleanup_server_process(args.pid2kill)
+                    self._cleanup_server_process(args.server_process)
                     raise e
                 logger.debug(f"Connection attempt failed, retrying... (attempt {i + 1}/{max_retries}): {e}")
                 time.sleep(3)
@@ -499,72 +536,20 @@ class Transport(TransportBase):
         except (socket.error, ValueError):
             return False
 
-    def _cleanup_server_process(self, pid):
+    def _cleanup_server_process(self, process):
         """Clean up the server process."""
-        if not pid:
+        if not process:
             return
 
-        import signal
-
+        process.terminate()
         try:
-            os.kill(pid, signal.SIGTERM)
-            time.sleep(1)  # Give it time to terminate gracefully
-            os.kill(pid, signal.SIGKILL)
-        except ProcessLookupError:
-            pass
+            process.wait(timeout=5)
+        except subprocess.TimeoutExpired:
+            process.kill()
 
-    def _build_mlx_exec_args(self, subcommand: str, args, extra: Optional[list[str]] = None) -> list[str]:
-        """Return the command-line *exec_args* for ``mlx_lm`` *subcommand*.
-        Parameters
-        ----------
-        subcommand:
-            Should just be ``"server"``
-        args:
-            Parsed CLI *args* namespace.
-        extra:
-            Optional list of extra arguments to append verbatim.
-        """
-        exec_args = [
-            "mlx_lm.server",
-            "--model",
-            shlex.quote(self._get_entry_model_path(args.container, args.generate, args.dryrun)),
-        ]
-
-        if getattr(args, "temp", None):
-            exec_args += ["--temp", str(args.temp)]
-
-        if getattr(args, "seed", None):
-            exec_args += ["--seed", str(args.seed)]
-
-        if getattr(args, "context", None):
-            exec_args += ["--max-tokens", str(args.context)]
-
-        exec_args += getattr(args, "runtime_args", [])
-
-        if extra:
-            exec_args += extra
-
-        return exec_args
-
-    def perplexity(self, args):
-        self.ensure_model_exists(args)
-        exec_args = self.build_exec_args_perplexity(args)
-        self.execute_command(exec_args, args)
-
-    def build_exec_args_perplexity(self, args):
-        if getattr(args, "runtime", None) == "mlx":
-            raise NotImplementedError("Perplexity calculation is not supported by the MLX runtime.")
-
-        # Default llama.cpp perplexity calculation
-        exec_args = ["llama-perplexity"]
+    def perplexity(self, args, cmd: list[str]):
         set_accel_env_vars()
-        gpu_args = self.gpu_args(args=args)
-        if gpu_args is not None:
-            exec_args.extend(gpu_args)
-
-        exec_args += ["-m", self._get_entry_model_path(args.container, False, args.dryrun)]
-
-        return exec_args
+        self.execute_command(cmd, args)
 
     def exists(self) -> bool:
         _, _, all = self.model_store.get_cached_files(self.model_tag)
@@ -577,25 +562,10 @@ class Transport(TransportBase):
             return
 
         if args.pull == "never":
-            raise ValueError(f"{args.MODEL} does not exists")
+            raise ValueError(f"{args.MODEL} does not exist")
 
         self.pull(args)
 
-    def build_exec_args_bench(self, args):
-        if getattr(args, "runtime", None) == "mlx":
-            raise NotImplementedError("Benchmarking is not supported by the MLX runtime.")
-
-        # Default llama.cpp benchmarking
-        exec_args = ["llama-bench"]
-        set_accel_env_vars()
-        gpu_args = self.gpu_args(args=args)
-        if gpu_args is not None:
-            exec_args.extend(gpu_args)
-
-        exec_args += ["-m", self._get_entry_model_path(args.container, False, args.dryrun)]
-
-        return exec_args
-
     def validate_args(self, args):
         # MLX validation
         if getattr(args, "runtime", None) == "mlx":
@@ -620,129 +590,6 @@ class Transport(TransportBase):
 
         raise KeyError("--nocontainer and --name options conflict. The --name option requires a container.")
 
-    def vllm_serve(self, args):
-        exec_args = [
-            "--model",
-            self._get_entry_model_path(args.container, args.generate, args.dryrun),
-            "--port",
-            args.port,
-            "--max-sequence-length",
-            f"{args.context}",
-        ]
-        exec_args += args.runtime_args
-        return exec_args
-
-    def llama_serve(self, args):
-        exec_args = ["llama-server"]
-        draft_model_path = None
-        if self.draft_model:
-            draft_model = self.draft_model._get_entry_model_path(args.container, args.generate, args.dryrun)
-            draft_model_path = MNT_FILE_DRAFT if args.container or args.generate else draft_model
-
-        exec_args += [
-            "--port",
-            args.port,
-            "--model",
-            self._get_entry_model_path(args.container, args.generate, args.dryrun),
-            "--no-warmup",
-        ]
-        if not args.thinking:
-            exec_args += ["--reasoning-budget", "0"]
-        mmproj_path = self._get_mmproj_path(args.container, args.generate, args.dryrun)
-        if mmproj_path is not None:
-            exec_args += ["--mmproj", mmproj_path]
-        else:
-            exec_args += ["--jinja"]
-
-            chat_template_path = self._get_chat_template_path(args.container, args.generate, args.dryrun)
-            if chat_template_path is not None:
-                exec_args += ["--chat-template-file", chat_template_path]
-
-        if should_colorize():
-            exec_args += ["--log-colors", "on"]
-
-        exec_args += [
-            "--alias",
-            self.model,
-            "--temp",
-            f"{args.temp}",
-            "--cache-reuse",
-            f"{args.cache_reuse}",
-        ]
-        if args.context > 0:
-            exec_args += ["--ctx-size", f"{args.context}"]
-
-        exec_args += args.runtime_args
-
-        if draft_model_path:
-            exec_args += ['--model_draft', draft_model_path]
-
-        # Placeholder for clustering, it might be kept for override
-        rpc_nodes = os.getenv("RAMALAMA_LLAMACPP_RPC_NODES")
-        if rpc_nodes:
-            exec_args += ["--rpc", rpc_nodes]
-
-        if args.debug:
-            exec_args += ["-v"]
-
-        if getattr(args, "webui", "") == "off":
-            exec_args.extend(["--no-webui"])
-
-        if check_nvidia() or check_metal(args):
-            exec_args.extend(["--flash-attn", "on"])
-        return exec_args
-
-    def mlx_serve(self, args):
-        extra = ["--port", str(args.port), "--host", args.host]
-        return self._build_mlx_exec_args("server", args, extra)
-
-    def build_exec_args_serve(self, args):
-        if args.runtime == "vllm":
-            exec_args = self.vllm_serve(args)
-        elif args.runtime == "mlx":
-            exec_args = self.mlx_serve(args)
-        else:
-            exec_args = self.llama_serve(args)
-
-        if args.seed:
-            exec_args += ["--seed", args.seed]
-
-        return exec_args
-
-    def handle_runtime(self, args, exec_args):
-        set_accel_env_vars()
-
-        if args.runtime == "vllm":
-            vllm_max_model_len = 2048
-            if args.context:
-                vllm_max_model_len = args.context
-
-            exec_args.extend(
-                [
-                    "--max_model_len",
-                    str(vllm_max_model_len),
-                    "--served-model-name",
-                    self.model_name,
-                ]
-            )
-
-            if getattr(args, 'runtime_args', None):
-                exec_args.extend(args.runtime_args)
-        elif args.runtime == "mlx":
-            # MLX uses the exec_args from mlx_serve
-            pass
-        else:
-            gpu_args = self.gpu_args(args=args)
-            if gpu_args is not None:
-                exec_args.extend(gpu_args)
-
-            if args.container:
-                exec_args.extend(["--host", "0.0.0.0"])
-            else:
-                exec_args.extend(["--host", args.host])
-
-        return exec_args
-
     def generate_container_config(self, args, exec_args):
         # Get the blob paths (src) and mounted paths (dest)
         model_src_path = self._get_entry_model_path(False, False, args.dryrun)
@@ -763,7 +610,7 @@ class Transport(TransportBase):
             )
         elif args.generate.gen_type == "kube":
             self.kube(
-                (model_src_path, model_dest_path),
+                (model_src_path.removeprefix("oci://"), model_dest_path),
                 (chat_template_src_path, chat_template_dest_path),
                 (mmproj_src_path, mmproj_dest_path),
                 args,
@@ -804,42 +651,35 @@ class Transport(TransportBase):
                 )
             raise NotImplementedError(file_not_found % {"cmd": exec_args[0], "error": str(e).strip("'")})
 
-    def serve(self, args, quiet=False):
-        self.ensure_model_exists(args)
-
-        args.port = compute_serving_port(args, quiet=quiet or args.generate)
-
-        exec_args = self.build_exec_args_serve(args)
-        exec_args = self.handle_runtime(args, exec_args)
+    def serve(self, args, cmd: list[str]):
+        set_accel_env_vars()
 
         if args.generate:
-            self.generate_container_config(args, exec_args)
+            self.generate_container_config(args, cmd)
             return
 
-        # Add rag chatbot
-        if getattr(args, "rag", None):
-            exec_args = [
-                "bash",
-                "-c",
-                f"nohup {' '.join(exec_args)} &> /tmp/llama-server.log & rag_framework run /rag/vector.db",
-            ]
-
-        self.execute_command(exec_args, args)
+        try:
+            self.execute_command(cmd, args)
+        except Exception as e:
+            self._cleanup_server_process(args.server_process)
+            raise e
 
     def quadlet(self, model_paths, chat_template_paths, mmproj_paths, args, exec_args, output_dir):
-        quadlet = Quadlet(self.model_name, model_paths, chat_template_paths, mmproj_paths, args, exec_args)
+        quadlet = Quadlet(
+            self.model_name, model_paths, chat_template_paths, mmproj_paths, args, exec_args, self.artifact
+        )
         for generated_file in quadlet.generate():
             generated_file.write(output_dir)
 
     def quadlet_kube(self, model_paths, chat_template_paths, mmproj_paths, args, exec_args, output_dir):
-        kube = Kube(self.model_name, model_paths, chat_template_paths, mmproj_paths, args, exec_args)
+        kube = Kube(self.model_name, model_paths, chat_template_paths, mmproj_paths, args, exec_args, self.artifact)
         kube.generate().write(output_dir)
 
-        quadlet = Quadlet(kube.name, model_paths, chat_template_paths, mmproj_paths, args, exec_args)
+        quadlet = Quadlet(kube.name, model_paths, chat_template_paths, mmproj_paths, args, exec_args, self.artifact)
         quadlet.kube().write(output_dir)
 
     def kube(self, model_paths, chat_template_paths, mmproj_paths, args, exec_args, output_dir):
-        kube = Kube(self.model_name, model_paths, chat_template_paths, mmproj_paths, args, exec_args)
+        kube = Kube(self.model_name, model_paths, chat_template_paths, mmproj_paths, args, exec_args, self.artifact)
         kube.generate().write(output_dir)
 
     def compose(self, model_paths, chat_template_paths, mmproj_paths, args, exec_args, output_dir):
@@ -859,62 +699,51 @@ class Transport(TransportBase):
         get_field: str = "",
         as_json: bool = False,
         dryrun: bool = False,
-    ) -> None:
+    ) -> Any:
         model_name = self.filename
         model_registry = self.type.lower()
-        model_path = self._get_entry_model_path(False, False, dryrun)
-
+        model_path = self._get_inspect_model_path(dryrun)
         if GGUFInfoParser.is_model_gguf(model_path):
             if not show_all_metadata and get_field == "":
                 gguf_info: GGUFModelInfo = GGUFInfoParser.parse(model_name, model_registry, model_path)
-                print(gguf_info.serialize(json=as_json, all=show_all))
-                return
+                return gguf_info.serialize(json=as_json, all=show_all)
 
             metadata = GGUFInfoParser.parse_metadata(model_path)
             if show_all_metadata:
-                print(metadata.serialize(json=as_json))
-                return
+                return metadata.serialize(json=as_json)
             elif get_field != "":  # If a specific field is requested, print only that field
                 field_value = metadata.get(get_field)
                 if field_value is None:
                     raise KeyError(f"Field '{get_field}' not found in GGUF model metadata")
-                print(field_value)
-                return
+                return field_value
 
         if SafetensorInfoParser.is_model_safetensor(model_name):
             safetensor_info: SafetensorModelInfo = SafetensorInfoParser.parse(model_name, model_registry, model_path)
-            print(safetensor_info.serialize(json=as_json, all=show_all))
-            return
+            return safetensor_info.serialize(json=as_json, all=show_all)
 
-        print(ModelInfoBase(model_name, model_registry, model_path).serialize(json=as_json))
+        return ModelInfoBase(model_name, model_registry, model_path).serialize(json=as_json)
 
-    def print_pull_message(self, model_name):
+    def print_pull_message(self, model_name) -> None:
         model_name = trim_model_name(model_name)
         # Write messages to stderr
         perror(f"Downloading {model_name} ...")
         perror(f"Trying to pull {model_name} ...")
 
-
-def distinfo_volume():
-    dist_info = "ramalama-%s.dist-info" % version()
-    path = os.path.join(os.path.dirname(os.path.dirname(__file__)), dist_info)
-    if not os.path.exists(path):
-        return ""
-
-    return f"-v{path}:/usr/share/ramalama/{dist_info}:ro"
+    def is_artifact(self) -> bool:
+        return False
 
 
-def compute_ports() -> list:
-    first_port = DEFAULT_PORT_RANGE[0]
-    last_port = DEFAULT_PORT_RANGE[1]
-    ports = list(range(first_port + 1, last_port + 1))
+def compute_ports(exclude: list[str] | None = None) -> list[int]:
+    excluded = exclude and set(map(int, exclude)) or set()
+    ports = list(sorted(set(range(DEFAULT_PORT_RANGE[0], DEFAULT_PORT_RANGE[1] + 1)) - excluded))
+    first_port = ports.pop(0)
     random.shuffle(ports)
     # try always the first port before the randomized others
     return [first_port] + ports
 
 
-def get_available_port_if_any() -> int:
-    ports = compute_ports()
+def get_available_port_if_any(exclude: list[str] | None = None) -> int:
+    ports = compute_ports(exclude=exclude)
     with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
         chosen_port = 0
         for target_port in ports:
@@ -929,13 +758,13 @@ def get_available_port_if_any() -> int:
         return chosen_port
 
 
-def compute_serving_port(args, quiet=False) -> str:
+def compute_serving_port(args, quiet: bool = False, exclude: list[str] | None = None) -> str:
     # user probably specified a custom port, don't override the choice
-    if getattr(args, "port", "") not in ["", str(DEFAULT_PORT)]:
+    if hasattr(args, 'port_override'):
         target_port = args.port
     else:
         # otherwise compute a random serving port in the range
-        target_port = get_available_port_if_any()
+        target_port = get_available_port_if_any(exclude=exclude)
 
     if target_port == 0:
         raise IOError("no available port could be detected. Please ensure you have enough free ports.")
